#include <userint.h>
#include <ansi_c.h>
//==============================================================================
//
// Title:       XEM.c
// Purpose:     A short description of the implementation.
//
// Created on:  1/13/2013 at 12:51:30 PM by Tom Burke.
// Copyright:   NG. All Rights Reserved.
//
//==============================================================================

//==============================================================================
// Include files
#include <windows.h>
#include "XEM.h"
#include "okFrontPanelDLL.h"
#include "globals.h"
#include "LogicAnalyzer.h"
#include <stdio.h>

//==============================================================================
// Constants

//==============================================================================
// Types

//==============================================================================
// Static global variables

//==============================================================================
// Static functions

//==============================================================================
// Global variables

//==============================================================================
// Global functions

//	Connects the software to the correct OK Board
int XEM_Connect ( FP_H* my_FP )
{
	int				NoDevices = 0;		//	Number of devices attached to the PC
	char			serial[50];			//	Device Serial number
	int				Device_number = 0;	//	The device we want to open
	
	
	//	Get a handle to the FrontPanel library
	my_FP->FP_Handle = okFrontPanel_Construct(  );
				
	//	Find out how many devices are attached
	NoDevices = okFrontPanel_GetDeviceCount( my_FP->FP_Handle );
	printf("%d OK devices attached\n", NoDevices);
	
	//	Make sure there's a board attached
	if( NoDevices < 1 )
	{
		printf("ERROR:\tNo XEM Devices attached!\n");
		return XEM_FAILURE;
	}
	
	
	//	Get the device to open
	if( NoDevices != 1 )
		my_FP->DeviceNum = XEM_GetDevice( my_FP, NoDevices );  //	More than one device, we need to have the user select the correct one
	else
		my_FP->DeviceNum = 0;	//	Only one device, we'll open this one


	//	Get the serial number of the device and copy it to the pointer info
	okFrontPanel_GetDeviceListSerial( my_FP->FP_Handle, Device_number, serial );
	strcpy( my_FP->SerNum, serial );
	printf("Opening Opal Kelly device number %d, Ser No %s\n", Device_number, serial );
	
	//	open the device		
	okFrontPanel_OpenBySerial( my_FP->FP_Handle, my_FP->SerNum );
	
    return XEM_SUCCESS;
}


//	Disconnects from the Frontpanel interface and device(s)
int XEM_Disconnect ( FP_H* my_FP )
{
	
	//	Call the Destructor (?)
	okFrontPanel_Destruct(my_FP->FP_Handle);    
	return XEM_SUCCESS;
}


//	Have the user choose the device to open
//	For future implementation - only have 1 board right now...
int XEM_GetDevice( FP_H* my_FP, int NoDevices )
{
//	int		count = 0;					//	Generic counter
//	char	List[150];					//	Contains parts of the selector list

	
//	//	Create a list of attached devices
//	for( count=0 ; count<NoDevices ; count++ )
//	{
		//	Get the board info for the list
//		XEM_CreateListEntry( FP_Handle, count, List[count] );
//		printf("Device %d is an Opal Kelly type %d,Ser No %s\n", count, Board_model, string );
//		
//		//	Insert the list entry into the list
//		InsertListItem( GUI_Panel, PANEL_2_LISTBOX,	count, List );
//	}
//	
//	//	List created - have the user select
//	DisplayPanel( PANEL_2 );

	return 0;	//	For now, ujst return device zero
}

//	Create the chooser list
int XEM_GetDeviceType( FP_H* my_FP, int Device_num )
{
	ok_BoardModel	Board_model = ok_brdUnknown;	//	This is an enumerated type
	char			serial[50];						//	Board Serial Number
	char			Board_type[20];					//	String for the board type
	
	//	Get the board model & serial number
	Board_model = okFrontPanel_GetDeviceListModel( my_FP->FP_Handle, Device_num );
	okFrontPanel_GetDeviceListSerial( my_FP->FP_Handle, Device_num, serial );
	
	//	Give test to the board type
	switch (Board_model)
	{
		case ok_brdXEM3001v1: 
				strcpy( Board_type, "XEM3001v1" );
				break;
		case ok_brdXEM3001v2: 
				strcpy( Board_type, "XEM3001v2" );
				break;
		case ok_brdXEM3010: 
				strcpy( Board_type, "XEM3010" );
				break;
		case ok_brdXEM3005: 
				strcpy( Board_type, "XEM3005" );
				break;
		case ok_brdXEM3001CL: 
				strcpy( Board_type, "XEM3001CL" );
				break;
		case ok_brdXEM3020: 
				strcpy( Board_type, "XEM3020" );
				break;
		case ok_brdXEM3050: 
				strcpy( Board_type, "XEM3050" );
				break;
		case ok_brdXEM9002: 
				strcpy( Board_type, "XEM9002" );
				break;
		case ok_brdXEM3001RB: 
				strcpy( Board_type, "XEM3001RB" );
				break;
		case ok_brdXEM5010: 
				strcpy( Board_type, "XEM5010" );
				break;
		case ok_brdXEM6110LX45: 
				strcpy( Board_type, "XEM6110LX45" );
				break;
		case ok_brdXEM6001: 
				strcpy( Board_type, "XEM6001" );
				break;
		case ok_brdXEM6010LX45: 
				strcpy( Board_type, "XEM6010LX45" );
				break;
		case ok_brdXEM6010LX150: 
				strcpy( Board_type, "XEM6010LX150" );
				break;
		case ok_brdXEM6110LX150: 
				strcpy( Board_type, "XEM6110LX150" );
				break;
		case ok_brdXEM6006LX9: 
				strcpy( Board_type, "XEM6006LX9" );
				break;
		case ok_brdXEM6006LX16: 
				strcpy( Board_type, "XEM6006LX16" );
				break;
		case ok_brdXEM6006LX25: 
				strcpy( Board_type, "XEM6006LX25" );
				break;
		case ok_brdXEM5010LX110: 
				strcpy( Board_type, "XEM5010LX110" );
				break;
		case ok_brdZEM4310: 
				strcpy( Board_type, "ZEM4310" );
				break;
		case ok_brdXEM6310LX45: 
				strcpy( Board_type, "XEM6310LX45" );
				break;
		case ok_brdXEM6310LX150: 
				strcpy( Board_type, "XEM6310LX150" );
				break;
		case ok_brdXEM6110v2LX45: 
				strcpy( Board_type, "XEM6110v2LX45" );
				break;
		case ok_brdXEM6110v2LX150: 
				strcpy( Board_type, "XEM6110v2LX150" );
				break;
		case ok_brdXEM6002LX9: 
				strcpy( Board_type, "XEM6002LX9" );
				break;
		case ok_brdXEM6310MTLX45T: 
				strcpy( Board_type, "XEM6310MTLX45T" );
				break;
		case ok_brdXEM6320LX130T: 
				strcpy( Board_type, "XEM6320LX130T" );
				break;
		default:
				strcpy( Board_type, "Unknown" );
	}

	//	Copy pertinent information to the FrontPanel pointer
	strcpy( my_FP->BoardModelStr, Board_type );
	my_FP->BoardModel = Board_model;
	
	
	
	
	return XEM_SUCCESS;
}


int CVICALLBACK AutomaticUpdateThreadFunction (void* my_FP)
{
	FP_Ptr	temp_ptr = (FP_Ptr)my_FP;

	while( !temp_ptr->Quit )
	{
		
		while (temp_ptr->Running) 
		{
			if( temp_ptr->Update == XEM_TRUE )
			{
				//	Set the WireIn value in the FrontPanel Interface
				okFrontPanel_SetWireInValue( temp_ptr->FP_Handle, 0x00, temp_ptr->WireIn[0x00], 0xffffffff );
			
				//	Send them TO the FPGA
				okFrontPanel_UpdateWireIns( temp_ptr->FP_Handle );
				
				//	Set the update flag bag to false
				temp_ptr->Update = XEM_FALSE;
			}
			
			//	Get *ALL* the WireOut Values FROM the FPGA
			okFrontPanel_UpdateWireOuts( temp_ptr->FP_Handle );
				
			//	Get the WireOut value from the FrontPanel interface
			temp_ptr->WireOut[0x00] = okFrontPanel_GetWireOutValue( temp_ptr->FP_Handle, 0x20 ); 
			
//			printf("Value = %x\n", temp_ptr->WireOut[0x00] );
			
		}
	}
    return 0;
}

int	WaitUpdate( FP_H* my_FP )			//	Wait for the update to complete
{
	while(my_FP->Update == XEM_TRUE){};					//	Do nothing for a bit
	
	return SUCCESS;
}


int ResetFIFOs( FP_H* my_FP )			//	Reset the memory controller and FIFOs
{
	unsigned int	temp=0;					//	working memory
	
	temp = my_FP->WireIn[0x00];				//	Get the current value of the WireINs

	temp = temp | 0x0002;					//  Fifo Reset is bit 1
	my_FP->WireIn[0x00] = temp;				//	Update the structure
	my_FP->Update = XEM_TRUE;				//	Tell the thread to send an update to the XEM board
	
	WaitUpdate( my_FP );					//	Wait for the update to complete

	temp = temp & 0xfffd;					//  Release the FIFO reset
	my_FP->WireIn[0x00] = temp;				//	Update the structure
	my_FP->Update = XEM_TRUE;				//	Tell the thread to send an update to the XEM board
	
	WaitUpdate( my_FP );					//	Wait for the update to complete
	
	
	
	return XEM_SUCCESS;
}

int StartAcq( FP_H* my_FP )				//	Set the hardware to start acquisition
{
	unsigned int	temp=0;					//	working memory
	
	temp = my_FP->WireIn[0x00];				//	Get the current value of the WireINs

	temp = temp | 0x0004;					//  Acquire signal is bit 2 - set it to "1"
	my_FP->WireIn[0x00] = temp;				//	Update the structure
	my_FP->Update = XEM_TRUE;				//	Tell the thread to send an update to the XEM board
	
	WaitUpdate( my_FP );					//	Wait for the update to complete
	
	return XEM_SUCCESS;
}
	

int StopAcq( FP_H* my_FP )				//	Set the hardware to start acquisition
{
	unsigned int	temp=0;					//	working memory
	
	temp = my_FP->WireIn[0x00];				//	Get the current value of the WireINs

	temp = temp & 0xFFFB;					//  Acquire signal is bit 2 - set it to "0"
	my_FP->WireIn[0x00] = temp;				//	Update the structure
	my_FP->Update = XEM_TRUE;				//	Tell the thread to send an update to the XEM board
	
	WaitUpdate( my_FP );					//	Wait for the update to complete
	
	return XEM_SUCCESS;
}

//	Check the XEM memory_full status bit
//	set mem_full to XEM_TRUE when the status bit is "1"
int CheckMemStatus( FP_H* my_FP,  int* mem_full )
{
	unsigned int	temp=0;					//	working memory
	
	temp = my_FP->WireOut[0x00];			//	Get the current value of the WireINs

	temp = temp & 0x0004;					//  mem_full signal is bit 2

	if( temp )
		*mem_full = 1;
	else
		*mem_full = 0;
	
	return XEM_SUCCESS;
}

//	Use the okPipe functionality to do a high-speed retrieval of 
//	32MB of memory from the XEM card.
//	This function assumes two things
//		1) LA_Memory pointer is big enough to not overflow
//		2) Memory has valid contents.
int RetrieveData( FP_H* my_FP, unsigned char *LA_Memory )	//	Get the data from the XEM card
{

	unsigned int	temp=0;					//	working memory
	int				BufferSize = 16*1024*1024-2;
	
	temp = my_FP->WireIn[0x00];				//	Get the current value of the WireINs

	temp = temp | 0x0001;					//  Read signal is bit 1 - set it to "1"
	my_FP->WireIn[0x00] = temp;				//	Update the structure
	my_FP->Update = XEM_TRUE;				//	Tell the thread to send an update to the XEM board
	
	WaitUpdate( my_FP );					//	Wait for the update to complete
	
	//	Hardware is ready to give us the memory from SDRAM, so now we need to open the pipe
	//	Pipe reads are limited to just shy of 16MB - will need to do two reads
		my_FP->Running = FALSE;		//	Stop the update thread so it doesn't collide
		//	Read the 1st half
		okFrontPanel_ReadFromPipeOut( my_FP->FP_Handle, 0xA0, BufferSize, LA_Memory);
		okFrontPanel_ReadFromPipeOut( my_FP->FP_Handle, 0xA0, BufferSize, LA_Memory+BufferSize);
		my_FP->Running = TRUE;		//	restart the update thread
	
	return XEM_SUCCESS;
	
}
