#include <ansi_c.h>
#include <utility.h>
#include <cvirte.h>		
#include <userint.h>
#include "LogicAnalyzer.h"
#include "globals.h"
#include "okFrontPanelDLL.h"

static int panelHandle;

int __stdcall WinMain (HINSTANCE hInstance, HINSTANCE hPrevInstance,
                       LPSTR lpszCmdLine, int nCmdShow)
{
	if (InitCVIRTE (hInstance, 0, 0) == 0)
		return -1;	/* out of memory */
	
	//	Allocate acquisition memory (32MB)
	LA_Memory = (short int*)malloc( sizeof(short int)*BUFFER_SIZE );
	
	if ((panelHandle = LoadPanel (0, "LogicAnalyzer.uir", PANEL)) < 0)
		return -1;
	DisplayPanel (panelHandle);
	RunUserInterface ();
	DiscardPanel (panelHandle);
	return 0;
}

int CVICALLBACK RunStop (int panel, int control, int event,
		void *callbackData, int eventData1, int eventData2)
{
	int	mem_full = 0;
	
	switch (event)
	{
		case EVENT_COMMIT:
			//	Get Control - are we starting or stopping an acquisition?
			GetCtrlVal( panel, control, &run_stop );

			//	Set zoom & scroll back to zero
			zoom_level = 1;						//	No zoom
			start_point = 0;					//	Start point offset = 0								
			start_point_ptr = LA_Memory;		//	Start point pointer set to beginning of acquisition memory
			display_len = BUFFER_SIZE;			//	reset to display all the points
			
			switch (run_stop)
			{
				//	Perform an acquisition
				case RUN:
					//	Disable the controls
					SetCtrlAttribute( PANEL, PANEL_SAMPLERATE, ATTR_DIMMED, TRUE );
					SetCtrlAttribute( PANEL, PANEL_Trigger, ATTR_DIMMED, TRUE );
					SetCtrlAttribute( PANEL, PANEL_ZOOM, ATTR_DIMMED, TRUE );
					SetCtrlAttribute( PANEL, PANEL_SCROLL, ATTR_DIMMED, TRUE );

					
					ResetFIFOs( &my_FP );						//	Reset the memory controller and FIFOs
					StartAcq( &my_FP );							//	Set the hardware to start acquisition
																		//	If set to ext trig, acq  won't start 
																		//	until trigger is RX'd
					//	Wait for XEM memory to be filled
					CheckMemStatus( &my_FP, &mem_full );		//	Check memory status
					while( !mem_full && (run_stop == RUN) )
					{
						CheckMemStatus( &my_FP, &mem_full );	//	Check memory status
						ProcessSystemEvents();							//	Don't be a hog
					}
					
					//	Memory is full, transfer it to PC
					StopAcq( &my_FP );						//	Turn off the acquisition
					ResetFIFOs( &my_FP );					//	Reset the memory controller and FIFOs
					RetrieveData( &my_FP, (unsigned char *)LA_Memory );		//	Get the data from the XEM card
					
					PlotPackedDigitalLines(panel, PANEL_DIGGRAPH, LA_Memory, BUFFER_SIZE, VAL_SHORT_INTEGER, &gMask);
					ResetFIFOs( &my_FP );					//	Reset the memory controller and FIFOs
				
					//	Acquisition is complete - set the button back to the run state
					SetCtrlVal( panel, control, OFF );
					
					//	Enable the controls
					SetCtrlAttribute( PANEL, PANEL_SAMPLERATE, ATTR_DIMMED, FALSE );
					SetCtrlAttribute( PANEL, PANEL_Trigger, ATTR_DIMMED, FALSE );
					SetCtrlAttribute( PANEL, PANEL_ZOOM, ATTR_DIMMED, FALSE );
					break;
					
				//	Stop an acquisition that's in progress
				case STOP:
					run_stop = STOP;

					break;
					
				default: printf("How did we get here?  ERROR in RunStop function!\n");
					return 0;
			}
			
			break;
	}
	
	return 0;
}

int CVICALLBACK Set_Sample_Rate (int panel, int control, int event,
		void *callbackData, int eventData1, int eventData2)
{	
	int	SampleRate;

	switch (event)
	{
		case EVENT_COMMIT:
			//	Get the logic analyzer sample rate settings
			GetCtrlVal( panel, PANEL_SAMPLERATE, &SampleRate );
			
			//	Put the sample rate on the opal kelly stuff
			//		Sample rate settings are bit [5:3]
				//	Set the values in our pointer - breaking this into two pieces for readability...
					//	Clear the old mux values out
					my_FP.WireIn[0] = (my_FP.WireIn[0] & 0xffc7);

					//	Set the new values
					my_FP.WireIn[0] = my_FP.WireIn[0] | ((SampleRate<<3) & 0x0038);
				
				//	Tell the monitor thread to update the controls
				my_FP.Update = XEM_TRUE;

			break;
	}
	return 0;
}

int CVICALLBACK Quit (int panel, int control, int event,
		void *callbackData, int eventData1, int eventData2)
{
	switch (event)
	{
		case EVENT_COMMIT:
			QuitUserInterface (0);
			break;
	}
	return 0;
}

int CVICALLBACK Connect (int panel, int control, int event,
		void *callbackData, int eventData1, int eventData2)
{
	int		Value = 0;
	int		Source = 0;
	char	infile[512] = ".\\logicanalyzer.bit";	//	file name & path to load

	
	switch (event)
	{
		case EVENT_COMMIT:
			GetCtrlVal(PANEL, PANEL_CONNECT, &Value);
			
			if( Value )
			{
				// Load the FrontPanel DLL
				if (FALSE == okFrontPanelDLL_LoadLib(NULL)) 
				{
					printf("Could not load FrontPanel DLL\n");
					//  Set the switch state bck to disconnected
					Value = 0;
					GetCtrlVal(PANEL, PANEL_CONNECT, &Value);
					return XEM_FAILURE;
				}

				//	Bail on failure
				if( XEM_Connect(&my_FP) != XEM_SUCCESS )
				{
					printf("XEM_Connect failed!\n");
					//  Set the switch state bck to disconnected
					Value = 0;
					GetCtrlVal(PANEL, PANEL_CONNECT, &Value);
					return XEM_FAILURE;
				}
				
				//	Load the bitfile to the FPGA
				//	Still need to do error checking...
				okFrontPanel_ConfigureFPGA( my_FP.FP_Handle, infile );

				
				//	Enable the controls
				SetCtrlAttribute( PANEL, PANEL_RunStop, ATTR_DIMMED, FALSE );
				SetCtrlAttribute( PANEL, PANEL_SAMPLERATE, ATTR_DIMMED, FALSE );
				SetCtrlAttribute( PANEL, PANEL_Trigger, ATTR_DIMMED, FALSE );
				
				
				//	Start the WireIn/Out monitor thread running
				my_FP.Running = Value;
				my_FP.Quit = FALSE;
				
				//	Start the thread
			    CmtScheduleThreadPoolFunction (DEFAULT_THREAD_POOL_HANDLE, AutomaticUpdateThreadFunction, &my_FP, &functionID);

				//	Get the trigger source from the GUI and set things up appropriately
				SetTriggerSource( panel, control, &my_FP ); 

				//	Set the sample rate to 50MHz
				my_FP.WireIn[0] = 0x0028;
				my_FP.Update = XEM_TRUE;

			}
			else
			{
				//	Disable the controls
				SetCtrlAttribute( PANEL, PANEL_RunStop, ATTR_DIMMED, TRUE );
				SetCtrlAttribute( PANEL, PANEL_SAMPLERATE, ATTR_DIMMED, TRUE );
				SetCtrlAttribute( PANEL, PANEL_Trigger, ATTR_DIMMED, TRUE );
				SetCtrlAttribute( PANEL, PANEL_ZOOM, ATTR_DIMMED, TRUE );
				SetCtrlAttribute( PANEL, PANEL_SCROLL, ATTR_DIMMED, TRUE );
				
				my_FP.Running = FALSE;		//	Send the signal to the Update Thread to stop running
				my_FP.Quit = TRUE;			//	Send the signal to the thread to exit

				//	Wait for the thread to quit
				CmtWaitForThreadPoolFunctionCompletion (DEFAULT_THREAD_POOL_HANDLE, functionID, THREAD_PRIORITY_NORMAL);	// Wait for LED Update thread to exit properly

				XEM_Disconnect(&my_FP);

				printf("\nXEM device disconnected\n");
				
			}
			

			break;
	}
	return 0;
}

int CVICALLBACK Trigger_Select (int panel, int control, int event,
		void *callbackData, int eventData1, int eventData2)
{
	int source;
	
	switch (event)
	{
		case EVENT_COMMIT:
				//	Get the trigger source from the GUI and set things up appropriately
				SetTriggerSource( panel, control, &my_FP ); 

			break;
	}
	return 0;
}

int	SetTriggerSource( int panel, int control, FP_H* my_FP )
{
	int	Source;
	
	//	Get the button state
	GetCtrlVal( panel, PANEL_Trigger, &Source );
	
	
	//	Set the trigger source for SW/HW
	if( Source == OFF )			//	Trigger comes from software
		TrigSource = INTERNAL;		//	Set it in SW
	else						//	Trigger comes from external source
		TrigSource = EXTERNAL;		//	Set it in SW

	
	SetTrigger( my_FP, TrigSource );	//	Set it in HW
	
	return TRUE;
}


int CVICALLBACK Scroll (int panel, int control, int event,
		void *callbackData, int eventData1, int eventData2)
{
	int direction = 0;
	int	scroll_count = 0;
	int	scroll_test = 0;
	
	switch (event)
	{
		case EVENT_COMMIT:
			GetCtrlVal( panel, PANEL_SCROLL, &direction );			//	Get the scroll direction
			SetCtrlVal( panel, PANEL_SCROLL, 0 );							//	re-center the control
				
			//	Determine how far to scroll (1/8th of the display)
			scroll_count = display_len/8;
			
			
			switch( direction )
			{
				case SCROLL_LEFT:	// scroll left
					
					//	Check to see if we can d the croll
					scroll_test = start_point - scroll_count;		//	Find the new start point
					
					if( scroll_test >= 0 )							//	If we can do it, 
						start_point = scroll_test;							//	Then do it
					
					break;
					
				case SCROLL_RIGHT:	//	scrol right
					
					// check for a valid scroll movement
					scroll_test = start_point + scroll_count;
					
					if( scroll_test < (BUFFER_SIZE-display_len) )
						start_point = scroll_test;
					
					break;
			}
			
			//	Find the start address
			start_point_ptr = LA_Memory + start_point;
					
			//	Display the data	
			PlotPackedDigitalLines(panel, PANEL_DIGGRAPH, start_point_ptr, display_len, VAL_SHORT_INTEGER, &gMask);

			break;
	}
	return 0;
}

int CVICALLBACK Zoom (int panel, int control, int event,
		void *callbackData, int eventData1, int eventData2)
{
	
	int	direction = 0;
	int	old_direction = 0;
	int	display_end = 0;
	
	
	switch (event)
	{
		case EVENT_COMMIT:
			SetCtrlAttribute( panel, PANEL_ZOOM, ATTR_DIMMED, TRUE );	//	turn off the zoom for a bit
			GetCtrlVal( panel, PANEL_ZOOM, &direction );				//	Get the control's value
			SetCtrlVal( panel, PANEL_ZOOM, 0 );							//	re-center the control

			old_direction = direction;			

				switch ( direction )
				{
					case D_ZOOM_OUT:	//	Zoom out

						if( zoom_level > 1  )
						{
							zoom_level--;									//	lower the zoom level
							display_len = BUFFER_SIZE/(pow(2,zoom_level-1));	//	decrease the number of points to display
						
							//	Figure out where to put the start of display pointer
								//	find the end of the display area based on current start point
								display_end = start_point + display_len;
							
								//	If the end is beyond the end of our data, then move the start point 
								//	far enoguh forward to be able to perform the zoom
								if( display_end > BUFFER_SIZE )
									start_point = BUFFER_SIZE - display_len;
						}
						break;
					
					case D_ZOOM_IN: 	//	Zoom in
						if( zoom_level < 20 )
						{
							zoom_level++;									//	Increase the zoom;
							display_len = BUFFER_SIZE/(pow(2,zoom_level-1));	//	decrease the number of points to display
						}
						break;
				}
					

				//	Find the start address
				start_point_ptr = LA_Memory + start_point;
					
				//	Display the data	
				PlotPackedDigitalLines(panel, PANEL_DIGGRAPH, start_point_ptr, display_len, VAL_SHORT_INTEGER, &gMask);
					
				//	enable/disable the scroller
				if( zoom_level == 0 )
					SetCtrlAttribute( panel, PANEL_SCROLL, ATTR_DIMMED, TRUE );
				else
					SetCtrlAttribute( panel, PANEL_SCROLL, ATTR_DIMMED, FALSE );
	
				SetCtrlAttribute( panel, PANEL_ZOOM, ATTR_DIMMED, FALSE );			//	re-enable zoom control
			
			break;
	}
	
	return 0;
}
