`timescale 1ns / 1ps
//////////////////////////////////////////////////////////////////////////////////
// Company: 
// Engineer: 
// 
// Create Date:    11:58:09 01/16/2013 
// Design Name: 
// Module Name:    ClockGen 
// Project Name: 
// Target Devices: 
// Tool versions: 
// Description: Takes 1 50MHz input clock and an external clock, and uses 3 select 
//						bits to choose an output of one of the following frequencies:
//						200MHz
//						100MHz
//						50MHz
//						20MHz
//						10MHz
//						5MHz
//						1MHz
//						EXT_CLK
//
// Dependencies: 
//
// Revision: 
// Revision 0.01 - File Created
// Additional Comments: 
//
//////////////////////////////////////////////////////////////////////////////////
module ClockGen(
    input CLK_IN,				//	Input CLK - expected to be 200MHz for this application
    input [2:0] CLK_SEL,	//	Output select - 3 bits to choose from 8 possibilities
    input EXT_CLK,			//	External clock source
	 output SA_CLK_OUT,		//	multiplexed sample clock output
	 output EXT_CLK_OUT		//	External cllock echoed back out for other things to use
    );

	wire			M_200;		//	(200MHz) - multiply by 4, divide by 1
	wire			M_100;		//	(100MHz) - multiply by 2
	wire			M_50;			//	(50MHz) - pass through
	wire			M_20;			//	(20MHz) - multiply by 2, divide by 5
	wire			M_10;			//	(10MHz) - divide by 5
	wire			M_5;			//	(5MHz) - divide by 10
	wire			M_1;			//	(1MHz) - derived from 20MHz
	wire			B_EXT_CLK;	//	EXT_CLK after it hits an IBUG
	wire			SEL_CLK;		//	The sample clock prior to buffering
	wire	[7:0] Clocks;		//	All clocks bussed together
	
	assign	Clocks[0] = B_EXT_CLK;
	assign	Clocks[1] = M_1;
	assign	Clocks[2] = M_5;
	assign	Clocks[3] = M_10;
	assign	Clocks[4] = M_20;
	assign	Clocks[5] = M_50;
	assign	Clocks[6] = M_100;
	assign	Clocks[7] = M_200;
	
	assign	EXT_CLK_OUT = B_EXT_CLK;
	

//--------------  Create our sample clocks via a counter
	//	define the register
	reg	[1:0]		ClockDiv;
	
	//	Assign the register bits to the appropriate clocks
	assign	M_200 = CLK_IN;
	assign	M_100 = ClockDiv[0];
	assign	M_50 = ClockDiv[1];
	
	
	always @(posedge CLK_IN)
		ClockDiv = ClockDiv + 1;

 
	//	Clock buffer for EXT_CLK
   IBUFG #(
      .IBUF_DELAY_VALUE("0"), // Specify the amount of added input delay for 
                              //   the buffer: "0"-"12" (Spartan-3E)
      .IOSTANDARD("DEFAULT")
   ) CLK_BUFF_EXT (
      .O(B_EXT_CLK), // Clock buffer output
      .I(EXT_CLK)  // Clock buffer input (connect directly to top-level port)
   );

	//	divide by 10 - output clock rate is 1/10th input clock rate (PRF)
	Div10 Create_20 (
		.CLK_IN(M_200),		//	using the 200MHz input clock 
		.CLK_OUT(M_20)			//		to synthesize a 20MHz clock for use in other places
		);
 
 	//	divide by 20 - output clock is 1/20th the input clock rate (PRF)
	Div20 Create_10 (
		.CLK_IN(M_200),		//	using the 200MHz clock from DCM1 as the input
		.CLK_OUT(M_10)			//		to synthesize a 10MHz clock for use in other places
		);

	//	divide by 20 - output clock is 1/20th the input clock rate (PRF)
	Div20 Create_5 (
		.CLK_IN(M_100),		//	using the 100MHz clock from DCM1 as the input
		.CLK_OUT(M_5)			//		to synthesize a 5MHz clock for use in other places
		);

	//	divide by 20 - output clock is 1/20th the input clock rate (PRF)
	Div20 Create_1 (
		.CLK_IN(M_20),			//	using the 20MHz clock from DCM1 as the input
		.CLK_OUT(M_1)			//		to synthesize a 1MHz clock for use in other places
		);

	//	mux the clocks
	ClockMux ClkMux(
		.CLK_IN( Clocks ),
		.CLK_SEL( CLK_SEL ),
		.MUX_OUT( SEL_CLK )
		);

	//  Buffer the clock on its way out
   BUFG BUFF_CLK_OUT (
      .O(SA_CLK_OUT),     	// Clock buffer output
      .I(SEL_CLK)      		// Clock buffer input
   );
endmodule
