`timescale 1ns / 1ps
//////////////////////////////////////////////////////////////////////////////////
// Company: 
// Engineer: 
// 
// Create Date:    13:21:03 01/16/2013 
// Design Name: 
// Module Name:    LogicAnalyzer 
// Project Name: 
// Target Devices: 
// Tool versions: 
// Description: 
//
// Dependencies: 
//
// Revision: 
// Revision 0.01 - File Created
// Additional Comments: 
//
//////////////////////////////////////////////////////////////////////////////////
module LogicAnalyzer(
	//	Board Interfaces
	input	wire				CLK_133MHz,			//	133MHz input clock from PLL
	input	wire				EXT_CLK,				//	External clock input
	input	wire	[15:0]	DataIn,				//	Parallel data input from the outside world
	output wire	[3:0]		led,					//	output to LEDs
	output wire				Sa_Clk_out,			//	Sample clock output to outside world

	//	Opal Kelly Interfaces
	input	wire	[7:0]		hi_in,				//	ok host interface input bus
	inout	wire	[15:0]	hi_inout,			//	ok host interface IO bus
	output wire	[1:0]		hi_out,				//	ok host interface output signalling
	output wire				hi_muxsel,
	output wire				i2c_sda,
	output wire				i2c_scl,

	//	SDRAM Interfaces
//	output wire				sdram_clk_out,
	output wire				sdram_cke,
	output wire				sdram_cs_n,
	output wire				sdram_we_n,
	output wire				sdram_cas_n,
	output wire				sdram_ras_n,
	output wire				sdram_lqdm,
	output wire				sdram_uqdm,
	output wire	[1:0]		sdram_ba,
	output wire	[12:0]	sdram_a,
	inout	wire	[15:0]	sdram_d
	);

	//	Set the I2C mux to the appropriate states
	assign	i2c_sda = 1'bz;
	assign	i2c_scl = 1'bz;
	assign	hi_muxsel = 1'b0;

	wire		GND_BIT;				//	This bit always logic "0"
	assign	GND_BIT = 1'b0;	//	Always 0
	
	wire		HI_BIT;				//	This bit always logic "1"
	assign	HI_BIT = 1'b1;		//	Always 1


//--------------------------------------------------------------------------------------
	//	Buffer the sdram_clk for internal use and also create our 200MHz master clock
	wire	w_sdram_clk;			//	133 MHz SDRAM clock - delay locked to input
	wire	w_sa_clk_source;		//	200MHz clock to the sample clock generator/mux
	wire	w_Mem_PLL_locked;		//	memory PLL locked signal
	
	
	dcm_sys	SDRAM_DCM(
		.i_CLK_IN(CLK_133MHz), 				//	133MHz input
		.i_RST_IN(GND_BIT),
		.o_CLK_IN_BUF(),						//	133MHz output (buffered by IBUFG)
		.o_CLK0_OUT(w_sdram_clk), 			//	133MHz output (Delay-locked to input)
		.o_CLK_FX(w_sa_clk_source),		//	200MHz output (no buffer)
		.o_LOCKED_OUT(w_Mem_PLL_locked)	//	DCM output locked signal
		);


//---------------------------------------------------------------------------------------
	//	Instantiate the Clock Generator 
	wire	[2:0] 	w_CLK_SEL;			//	wires into the mux
	wire				w_sample_clock;	//	This is the sample clock chosen by the mux
	assign	Sa_Clk_out = w_sample_clock;

	ClockGen Clock_Synth(
		.CLK_IN(w_sa_clk_source),		//	200MHz from PLL
		.CLK_SEL(w_CLK_SEL),				//	Clock select from PC
		.EXT_CLK(EXT_CLK),				//	External clock from ??
		.SA_CLK_OUT(w_sample_clock),	//	Sample clock output
		.EXT_CLK_OUT()	//	Echo of a buffered version of the external clock
		);
		
	
//--------------------------------------------------------------------------------------
	//	The Logic Analyzer input will run all the time.  
	//		Data collection is controlled via FIFO_wr_en and DRAM_wr_en
	wire	[15:0]		w_data_buffer;				//	Connection to the data buffer
//	reg	[15:0]		r_data_buffer;				//	Data buffer to DUT
	
//	assign	w_data_buffer = r_data_buffer;
	
//	//	Capture the data - 
//	always @(negedge w_sample_clock)
//		r_data_buffer = DataIn;				//	Clock data from DUT

	//  Capture data in a front-end FIFO.
	//		This allows us to have a fifo full (1024 samples) of collected data *before* the trigger arrives
	fifo_shift_16_1k input_fifo (
	  .clk(w_sample_clock),					// input clk - this is our sample clock
	  .rst(GND_BIT), 							// input rst - never gonna reset this...
	  .din(DataIn), 							// input [15 : 0] din
	  .wr_en(HI_BIT), 						// input wr_en - always enabled
	  .rd_en(HI_BIT), 						// input rd_en - always enabled
	  .dout(w_data_buffer), 				// output [15 : 0] dout - this is our buffered data
	  .full(), 									// output full - don't care
	  .empty() 									// output empty - don't care
	);
	
	
//-------------------------------------------------------------------------------------
	//	Instantiate the okInterfaces
	wire					ti_clk;					//	ti_clk from OK interface - use to read output fifo
	wire	[15:0]		wires_in;				//	Control signals from PC
	wire	[15:0]		wires_out;				//	Status signals to PC
	wire	[15:0]		w_pipeout;				//	High-speed data connection to PC
	wire					w_piperead;				//	read request signal from PipeOut
	
	okInterface LA_OK(
		.hi_in(hi_in),								//	Connections to Host Interface
		.hi_out(hi_out),
		.hi_inout(hi_inout),
		.ti_clk(ti_clk),							//	48MHz clock synch'd to USB connection
		.wires_in(wires_in),						//	Wires in FROM PC [15:0]
		.wires_out(wires_out),					//	Wires out TO PC [15:0]
		.pipe_out(w_pipeout),					//	high speed Data pipe TO PC [15:0]
		.pipe_read(w_piperead)					//	Pipe Read request (TO output FIFO)
		);

//--------------------------------------------------------------------------------------		
	//	break out some INPUT wires
	wire					w_perform_acq;		//	Perform an acquisition (enable writes to memory)
	wire					w_reset;				//	Reset memory signal
	wire					w_data_rd;			//	Read data from memory
	wire					w_trig_sel;			//	Trig_sel 0=internal, 1=external
	
	assign	{w_perform_acq, w_reset, w_data_rd} = wires_in[2:0];	//	Major Control Inputs
	assign	w_CLK_SEL = wires_in[5:3];										//	Sample Clock Select
	assign	w_trig_sel = wires_in[6];										//	Tell the HW which trigger source to use
	
	//	break out some OUTPUT wires
	wire					w_mem_full;				//	memory full (acq complete) signal
	wire					w_reading;				//	signal indicating we are reading from memory
	wire					w_writing;				//	signal that an acquisition is ongoing
	wire					w_fault_ifull;			//	error signal indicating the input FIFO has filled up (DUT read faster than SDRAm could handle)
	wire					w_fault_iempty;		//	error signal indicating SDRAM got starved (Transfer Negotiator should ensure this never happens)
	wire					w_fault_ofull;			//	error signal indicating the output FIFO is overflowing (Transfer Negotiator should ensure this never happens)
	wire					w_fault_oempty;		//	error signal indicating output FIFO is starved when SDRAM writing to it - should ever happen, as USB transfer is *much* slower than SDRAM

	assign	wires_out[9:0] = {w_fault_ifull, w_fault_iempty, w_fault_ofull, w_fault_oempty, 1'b0, w_piperead, w_Mem_PLL_locked, w_mem_full, w_reading, w_writing};
	assign	led = {~w_Mem_PLL_locked, ~w_mem_full, ~w_reading, ~w_writing};

//--------------------------------------------------------------------------------------
	//	Instantiate the trigger select
	wire		w_acquire_cmd;						//	command to memory to acquire data
	wire		w_ext_trig;							//	external trigger
	assign	w_ext_trig = DataIn[15];			//	is the MSB of our input data
	
	ManageTrigger	TrigSel(
		.i_trig_sel(w_trig_sel), 				//	0 = INTERNAL; 1 = EXTERNAL trigger
		.i_acq_cmd(w_perform_acq),				//	Write enable TO memory FROM PC 
		.i_ext_trig(w_ext_trig),				//	external trigger input 
		.i_reset(w_reset), 						//	reset signal
		.i_clk(w_sample_clock),					//	sample clock 
		.o_acquire(w_acquire_cmd)
		);
		
		
//--------------------------------------------------------------------------------------
	//	Instantiate the memory interface
	MemoryInterface	LA_mem(
		.DataIn(w_data_buffer),				//	input databus FROM UUT
		.DataOut(w_pipeout),					//	output data bus TO PC
		.clk_in(w_sdram_clk),				//	input clock - assume 50MHz
		.write_en(w_acquire_cmd),			//	Write enable TO memory FROM PC
		.write_clk(w_sample_clock),		//	write data valid at rising edge (basically, we're one behind in the pipeline)
		.read_en(w_data_rd),					//	enable reads FROM memory TO PC
		.read_req(w_piperead),				//	Tell output FIFO that USB is ready for data
		.read_clk(ti_clk),					//	requsted data should be valid on rising edge
		.reset(w_reset),						//	reset signal
		.mem_full(w_mem_full),				//	Signals that we've filled the memory
		.reading(w_reading),					//	Indicates that data is being read in FROM DUT
		.writing(w_writing),					//	Indicates that data is being written TO PC
													//	Direct SDRAM connections
		.sdram_clk_out(/*sdram_clk_out*/),	//	this is only for when the fabric supplies the clock - option not installed on our board!
		.sdram_cke(sdram_cke),
		.sdram_cs_n(sdram_cs_n),
		.sdram_we_n(sdram_we_n),
		.sdram_cas_n(sdram_cas_n),
		.sdram_ras_n(sdram_ras_n),
		.sdram_ldqm(sdram_lqdm),
		.sdram_udqm(sdram_uqdm),
		.sdram_ba(sdram_ba),
		.sdram_a(sdram_a),
		.sdram_d(sdram_d),
		.fault_ifull(w_fault_ifull),
		.fault_iempty(w_fault_iempty),
		.fault_ofull(w_fault_ofull),
		.fault_oempty(w_fault_oempty)
   );
	 


endmodule
