`timescale 1ns / 1ps
//////////////////////////////////////////////////////////////////////////////////
// Company: 
// Engineer: 
// 
// Create Date:    09:00:37 03/28/2013 
// Design Name: 
// Module Name:    Sample_DCM 
// Project Name: 
// Target Devices: 
// Tool versions: 
// Description: 
//
// Dependencies: 
//
// Revision: 
// Revision 0.01 - File Created
// Additional Comments: 
//
//////////////////////////////////////////////////////////////////////////////////
module Sample_DCM(
    input CLK_IN,
    input RESET_IN,
	 output CLK_RDY,
    output M_50,
    output M_100,
    output M_20,
    output M_200
    );

	wire		GND_BIT;				//	This wire is always a logic "0"
	assign	GND_BIT = 0;		//	always logic zero
	
	//	These wires are all used internally between the DCM and the BUFGs
	wire		I_M_50;
	wire		I_M_100;
	wire		I_M_20;
	wire		I_M_200;

	// DCM_SP: Digital Clock Manager Circuit
   //         Spartan-3E
   // Xilinx HDL Language Template, version 14.3
   DCM_SP #(
      .CLKDV_DIVIDE(2.5), 				// Divide by: 1.5,2.0,2.5,3.0,3.5,4.0,4.5,5.0,5.5,6.0,6.5
													//   7.0,7.5,8.0,9.0,10.0,11.0,12.0,13.0,14.0,15.0 or 16.0
      .CLKFX_DIVIDE(1),						// Can be any integer from 1 to 32
      .CLKFX_MULTIPLY(4),					// 200MHZ = 50MHz * 4/1
      .CLKIN_DIVIDE_BY_2("FALSE"),		// TRUE/FALSE to enable CLKIN divide by two feature
      .CLKIN_PERIOD(20),					// Specify period of input clock
      .CLKOUT_PHASE_SHIFT("NONE"),		// Specify phase shift of NONE, FIXED or VARIABLE
      .CLK_FEEDBACK("2X"),					// Specify clock feedback of NONE, 1X or 2X
      .DESKEW_ADJUST("SYSTEM_SYNCHRONOUS"),	// SOURCE_SYNCHRONOUS, SYSTEM_SYNCHRONOUS or
															//   an integer from 0 to 15
      .DLL_FREQUENCY_MODE("LOW"),		// HIGH or LOW frequency mode for DLL
      .DUTY_CYCLE_CORRECTION("TRUE"),	// Duty cycle correction, TRUE or FALSE
      .PHASE_SHIFT(0),						// Amount of fixed phase shift from -255 to 255
      .STARTUP_WAIT("FALSE")				// Delay configuration DONE until DCM LOCK, TRUE/FALSE
   ) DCM1 (
      .CLK0(I_M_50),			// 0 degree DCM CLK output
      .CLK180(),				// 180 degree DCM CLK output
      .CLK270(),				// 270 degree DCM CLK output
      .CLK2X(I_M_100),			// 2X DCM CLK output - 100MHz
      .CLK2X180(),			// 2X, 180 degree DCM CLK out
      .CLK90(),				// 90 degree DCM CLK output
      .CLKDV(I_M_20),			// Divided DCM CLK out (CLKDV_DIVIDE)
      .CLKFX(I_M_200),			// DCM CLK synthesis out (M/D) - 200MHz
      .CLKFX180(),			// 180 degree CLK synthesis out
      .LOCKED(CLK_RDY),	// DCM LOCK status output
      .PSDONE(),				// Dynamic phase adjust done output
      .STATUS(),				// 8-bit DCM status bits output
      .CLKFB(M_100),			// DCM clock feedback - 100MHz
      .CLKIN(CLK_IN),		// Clock input (from IBUFG, BUFG or DCM)
      .PSCLK(GND_BIT),		// Dynamic phase adjust clock input
      .PSEN(GND_BIT),		// Dynamic phase adjust enable input
      .PSINCDEC(GND_BIT),	// Dynamic phase adjust increment/decrement
      .RST(RESET_IN)			// DCM asynchronous reset input
   );

	//	Buffer the DCM outputs
   BUFG BUFG_20 (
      .O(M_20),     // Clock buffer output
      .I(I_M_20)      // Clock buffer input
   );

   BUFG BUFG_50 (
      .O(M_50),     // Clock buffer output
      .I(I_M_50)      // Clock buffer input
   );

   BUFG BUFG_100 (
      .O(M_100),     // Clock buffer output
      .I(I_M_100)      // Clock buffer input
   );

   BUFG BUFG_200 (
      .O(M_200),     // Clock buffer output
      .I(I_M_200)      // Clock buffer input
   );
	

endmodule
