`timescale 1ns / 1ps
//////////////////////////////////////////////////////////////////////////////////
// Company: 
// Engineer: 
// 
// Create Date:    16:29:11 01/21/2013 
// Design Name: 
// Module Name:    okInterface 
// Project Name: 
// Target Devices: 
// Tool versions: 
// Description: A standalone module for interface to the ok stuff.  Putting it in its own 
//				module to keep the next-level up code a little neater...
//
// Dependencies: 
//
// Revision: 
// Revision 0.01 - File Created
// Additional Comments: 
//
//////////////////////////////////////////////////////////////////////////////////
module okInterface(
    input 	[7:0]		hi_in,		//	Direct connect to USB stuff
    inout 	[15:0]	hi_inout,	//	Direct connect to USB stuff
    output 	[1:0]		hi_out,		//	Direct connect to USB stuff
	 output				ti_clk,		//	48MHz clock available to <whatever>
    output 	[15:0]	wires_in,	//	okWireIn (address 0x00) module for use <wherever> 
	 input 	[15:0]	wires_out,	//	okWireOut (address 0x20) module for use <wherever>
	 input	[15:0]	pipe_out,	//	okPipeOut (address 0xA0) modul for high speed data out
	 output				pipe_read	//	Write enable for the pipe out
    );

	//	target interface bus
	wire	[30:0]	ok1;			//	The input bus FROM the PC
	wire	[16:0]	ok2;			//	ok2 interface to the okHost from the okWireOR
	wire	[33:0]	ok2s;			//	input to the okWireOR
	
	//	Instantiate the okHost and connect endpoints
	okHost okHI(
		.hi_in(hi_in),				//	Connection to external pins
		.hi_out(hi_out),			//	Connection to externaql pins
		.hi_inout(hi_inout),		//	Connection to external pins
		.ti_clk(ti_clk),			//	We'll make this available to the next-higher module, just in case
		.ok1(ok1),					//	This is the bus coming FROM the PC
		.ok2(ok2)
		);
		
	//	Connect the ok host interface to our upper-level design
	//	Wires *IN*
	okWireIn	WireIN_00(
		.ok1(ok1),					//	Connection to the ok1 bus
		.ep_addr(8'h00),			//	This endpoint is at adress 0x00
		.ep_dataout(wires_in)	//	Connect these directly to the output
		);		
	
	//	Wires *OUT*
	okWireOut	WireOUT_20(
		.ok1(ok1),					//	Connection to the ok1 bus
		.ok2(ok2s[16:0]),					//	Connection to ok2 bus
		.ep_addr(8'h20),			//	This endpoint is at adress 0x20
		.ep_datain(wires_out)	//	Connect these directly to the input
		);

	//	Pipe *OUT*
	okPipeOut	PipeOUT_A0(
		.ok1(ok1),
		.ok2(ok2s[33:17]),		//	The pipeout is using the "top" 17 bits
		.ep_addr(8'hA0),			//	This endpoint is at address 0xA0
		.ep_read(pipe_read),		//	If this is a "1," then the pipe wants to read
		.ep_datain(pipe_out)		//	Dat going back to the PC
		);
	
	//	Need a WireOR to connect PipeOut and WireOut to PC
	okWireOR	#( .N(2) )
		WireOR(
			.ok2(ok2),
			.ok2s(ok2s)
			);
		

endmodule
